% compute the average precision of all methods, generate table 2 in the
% paper
clear all, clc
addpath(genpath(pwd)); 

vl_setup;
datasets = {'8'};         % '15', '61', '8', '101', 'holiday', 'ukbench'
data_dir = './data/';    
cheap_fet_name = {'lbp'};       %  'lbp', 'gist', 'phog' : concatination of all cheap features as the TFs
expe_fet_name = {'cnn_deep'};   %' cnn_deep', 'sift_llc', or 'ob': each single feature as one of the TFS
embedding_methods = {'NPE', 'Eigenmaps'}; 

train_rate = 0.5; 
across_class = 0;   % default to 1 for holiday and ukbench, as only very few images per class 

random_splits = 2;  % the number of random splits 

Purity = [];
for run_id = 1:random_splits 

%%
for ds_id=1:numel(datasets)
    
    dataset = datasets{ds_id};     
    switch dataset
        case 'holiday' 
            recall = 1; 
            across_class = 1;   %train & test sample manner
        case 'ukbench'
            recall = 1; 
            across_class = 1;   %train & test sample manner
        otherwise 
            recall = 0.1;
    end
%%  %% loading cheap features     
    cheap_fet = [];
    fet_num = length(cheap_fet_name);
    for m_id=1:fet_num
        fet_name_m = cheap_fet_name{m_id};     
        data_name = strcat( data_dir, 'data_', dataset, '_', fet_name_m, '.mat');        
        load(data_name);
        cheap_fet  =  cat(2, cheap_fet, data.fet);                    % the feature itself for ep  
        img_lab  =  data.lab(:);      
    end 

    %% sampling data: training vs. testing 
    img_num = length(img_lab(:)); 
    class_num = length(unique(img_lab));     

    if across_class
        class_index = randperm(class_num);
        train_c_ind = class_index(1:train_rate*class_num);
        train_ind = []; test_ind = 1:img_num;
        for cii=1:numel(train_c_ind)
           train_ind = cat(1, train_ind, find(img_lab==train_c_ind(cii))); 
        end
        test_ind(train_ind) = [];

    else 
        img_index = randperm(img_num);
        train_ind = img_index(1:train_rate*img_num);
        test_ind  = img_index(train_rate*img_num:end);  
    end

    train_lab = img_lab(train_ind); 
    test_lab  = img_lab(test_ind);
    test_num = numel(test_lab);
    train_num = numel(train_lab);
    %% MI

    % cheap features 
    train_fet_ch = cheap_fet(train_ind,:);
    test_fet_ch = cheap_fet(test_ind,:);
    
    switch cheap_fet_name{1};
        case 'gist' 
            ker_cheap = 'euclidean';
        otherwise 
            ker_cheap = 'chisq'; 
    end
        
    D_ch_te = pdist2(test_fet_ch, test_fet_ch, ker_cheap); 
    D_ch_te = real(D_ch_te);    

    for fid=1:numel(expe_fet_name)

        fet_name_m = expe_fet_name{fid};     
        data_name = strcat( data_dir, 'data_', dataset, '_', fet_name_m, '.mat');        
        load(data_name); data.fet = double(data.fet);
        train_fet_ex = data.fet(train_ind,:);
        test_fet_ex = data.fet(test_ind,:); 

        D_ex_tr = pdist2_large(train_fet_ex, train_fet_ex, 'euclidean');
        D_ex_te = pdist2_large(test_fet_ex, test_fet_ex, 'euclidean'); 
         
        D_ex_tr = real(D_ex_tr);
        D_ex_te = real(D_ex_te);

        for ml_id=1:numel(embedding_methods)
            embedding_method = embedding_methods{ml_id};

          if any(strcmpi(embedding_method, {'NPE', 'Eigenmaps'}));  % NPE, Eigenmaps, NCA, MLR, KISS
             [eigvector] = MetricImitation(train_fet_ch, train_fet_ex, D_ex_tr, embedding_method);
          else 
              error('The encoding method does not exist!');
          end

                %% projection for test feature        
            train_fet_pro = train_fet_ch * eigvector;
            test_fet_pro = test_fet_ch * eigvector ; 

            D_pro_te = pdist2(test_fet_pro, test_fet_pro, 'euclidean'); 
            D_pro_te = real(D_pro_te);


            %% retrieval evaluation 
            switch dataset
                case 'holiday'   % the first image of each group as the query
                    test_ulabs = unique(test_lab);
                    for n=1:numel(test_ulabs)
                        % cheap 
                        retrel_lab = test_lab == test_ulabs(n);  
                        n_inds = find(test_lab == test_ulabs(n)); q_ind = n_inds(1);   
                        retrel_lab(q_ind) = []; 
                        pos_n = (-normalize(D_ch_te(q_ind,:),2)); pos_n(q_ind) = [];
                        MapMatx(n, ds_id, 1, run_id) = MapCompt(retrel_lab, pos_n, recall, 1);

                        % mimic
                        pos_n = (-normalize(D_pro_te(q_ind,:),2)); pos_n(q_ind) = [];
                        MapMatx(n, ds_id, (fid-1)*3 + 1 + ml_id, run_id) = MapCompt(retrel_lab, pos_n, recall, 1);

                        % expensive
                        pos_n = (-normalize(D_ex_te(q_ind,:),2)); pos_n(q_ind) = [];
                        MapMatx(n, ds_id, (fid-1)*3 + 4, run_id) = MapCompt(retrel_lab, pos_n, recall, 1);
                       [n]
                    end  
                 
                otherwise
                    
                    for n=1:test_num
                        % cheap 
                        retrel_lab = test_lab == test_lab(n);  
                        retrel_lab(n) = [];
                        pos_n = exp(-normalize(D_ch_te(n,:),2)); pos_n(n) = [];
                        MapMatx(n, ds_id, 1, run_id)  = MapCompt(retrel_lab, pos_n, recall, 1);

                        % mimic
                        pos_n = exp(-normalize(D_pro_te(n,:),2)); pos_n(n) = [];
                        MapMatx(n, ds_id, (fid-1)*3 + 1 + ml_id, run_id) = MapCompt(retrel_lab, pos_n, recall, 1);

                        % expensive
                        pos_n = exp(-normalize(D_ex_te(n,:),2)); pos_n(n) = [];
                        MapMatx(n, ds_id, (fid-1)*3 + 4, run_id) = MapCompt(retrel_lab, pos_n, recall, 1);
                        [n]
                    end 
            end
        end %ml_id % over LEE and LPP
    end   %fid: over EFs
%     clear MapMatx
    
end % ds_id : over dataset

end
%% print results
MapMatx_MEAN = mean(MapMatx, 4); MapMatx_MEAN = MapMatx_MEAN(:,:,:);
MapMatx_MEAN = (mean(MapMatx_MEAN, 1));  MapMatx_MEAN = permute(MapMatx_MEAN, [2 3 1]);
purity_print(datasets, expe_fet_name, MapMatx_MEAN);

disp('finished...');
